unit UnitClient;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Gauges, ComCtrls, ToolWin, Sockets, ImgList, XPMan;

type
  TForm1 = class(TForm)
    Gauge1: TGauge;
    Memo1: TMemo;
    TcpClient1: TTcpClient;
    ToolBar1: TToolBar;
    ButtonConnect: TToolButton;
    EditHost: TEdit;
    lbl1: TLabel;
    EditPort: TEdit;
    lbl2: TLabel;
    OpenDialog1: TOpenDialog;
    Button1: TToolButton;
    ButtonSendFile: TToolButton;
    lbl3: TLabel;
    EditLine: TEdit;
    ButtonSendLine: TToolButton;
    Button2: TToolButton;
    Label4: TLabel;
    LabelLength: TLabel;
    Label1: TLabel;
    LabelBytes: TLabel;
    XPManifest1: TXPManifest;
    ImageList1: TImageList;
    procedure ButtonConnectClick(Sender: TObject);
    procedure TcpClient1Connect(Sender: TObject);
    procedure Display(const Msg : String);
    procedure TcpClient1Disconnect(Sender: TObject);
    procedure ButtonSendFileClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ButtonSendLineClick(Sender: TObject);
    procedure SendFile(FileName: string);
    procedure Progress(currval, maxval: Int64);
    procedure AbortConnection;
    procedure EditLineChange(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Controls(Enabled : boolean);
  private
    FFileStream : TFileStream;
    { Private declarations }
  public
    { Public declarations }
  end;

  
const
  CONST_BUFSIZE : Integer = 4096;
  MARKER_LINE : string[4] = 'line';
  MARKER_FILE : string[4] = 'file';


var
  Form1: TForm1;

implementation

{$R *.dfm}


procedure TForm1.FormShow(Sender: TObject);
begin
EditLineChange(Self);
Controls(False);
end;


procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
if TcpClient1.Active then ButtonConnect.Click;
end;



procedure TForm1.ButtonConnectClick(Sender: TObject);
begin
try
  if not TcpClient1.Connected then
    begin
      TcpClient1.Close;
      TcpClient1.RemoteHost := EditHost.Text;
      TcpClient1.RemotePort := EditPort.Text;
      TcpClient1.Open;
    end
  else TcpClient1.Disconnect;  

finally
  EditHost.Enabled := not TcpClient1.Connected;
  EditPort.Enabled := not TcpClient1.Connected;
  ButtonConnect.Down := TcpClient1.Connected;
  Controls(TcpClient1.Connected);
end;
end;



procedure TForm1.Display(const Msg : String);
begin
    Memo1.Lines.BeginUpdate;
    try
        Memo1.Lines.Add('['+TimeToStr(Time)+'] '+ Msg);
    finally
        Memo1.Perform(WM_VSCROLL, SB_BOTTOM, SB_THUMBTRACK);
        Memo1.Lines.EndUpdate;
    end;
end;




procedure TForm1.TcpClient1Connect(Sender: TObject);
begin
Display('Connected to ' + QuotedStr(TcpClient1.RemoteHost + ':' + TcpClient1.RemotePort));
end;


procedure TForm1.TcpClient1Disconnect(Sender: TObject);
begin
Display('Disconnected from ' + QuotedStr(TcpClient1.RemoteHost + ':' + TcpClient1.RemotePort));
end;




procedure TForm1.ButtonSendFileClick(Sender: TObject);
begin
if OpenDialog1.Execute then
  begin
     Display('  ' + QuotedStr(OpenDialog1.FileName));
     SendFile(OpenDialog1.FileName);
  end;
end;


procedure TForm1.ButtonSendLineClick(Sender: TObject);
var
  bt : integer;
begin
if Trim(EditLine.Text) <> '' then
  begin
     EditLine.Text := Trim(Editline.Text);
     Display('|------------------------------');
     Display('| Send string ' + QuotedStr(EditLine.Text));

     try
       TcpClient1.Sendln(MARKER_LINE);
       bt := TcpClient1.Sendln(EditLine.Text);
       Display('| Send bytes: ' + QuotedStr(IntToStr(bt)));
       Display('| Send length: ' + IntToStr(length(EditLine.Text)));
       Display('+------------------------------');
     except
       on E: Exception do begin Display('# ' + E.Message); end;
     end;
  end;
end;




procedure TForm1.SendFile(FileName: string);
const
  delim : string[1] = ':';
var
  buf : Pointer;
  nRead : Integer;
  markerstring : string;
begin
if not FileExists(FileName) then Exit;
if TcpClient1.Connected then
  begin
     try
       FFileStream := TFileStream.Create(FileName, fmOpenRead);
       FFileStream.Position := 0;
       Progress(0, 0);
       try
         //  "",     (      ...)
         FileName := ExtractFileName(FileName);
         //markerstring := MARKER_FILE + delim + FileName + delim + IntToStr(FFileStream.Size);
         Display('|------------------------------');
         Display('| Send marker string ' + QuotedStr(MARKER_FILE + delim + FileName + delim + IntToStr(FFileStream.Size)));
         //TcpClient1.Sendln(markerstring);
         TcpClient1.Sendln(MARKER_FILE);
         Display('| File name: ' + QuotedStr(FileName));
         TcpClient1.Sendln(FileName);
         Display('| File size: ' + QuotedStr(IntToStr(FFilestream.Size)));
         TcpClient1.Sendln(IntToStr(FFilestream.Size));
         Display('| Sending...');
       except
         on E: Exception do begin Display('# ' + E.Message); AbortConnection; Exit; end;
       end;

       // 
       repeat
         try
           if TcpClient1.Connected then
             begin
               GetMem(buf, CONST_BUFSIZE);

               nRead := FFileStream.Read(buf^, CONST_BUFSIZE);
               if nRead > 0 then
                  begin
                    try
                      TcpClient1.SendBuf(buf^, nRead)
                    except
                      on E: Exception do begin Display('# ' + E.Message); AbortConnection; Exit; end;
                    end;
                    Progress(FFileStream.Position, FFileStream.Size);
                  end;

               Application.ProcessMessages;
             end;
         finally
           FreeMem(buf, CONST_BUFSIZE);
         end;

       until nRead <= 0;

       Display('| File ' + QuotedStr(FileName) + ' sent');
       Display('+------------------------------');
     finally
       if FFileStream <> nil then begin FFileStream.Free; FFileStream := nil; end;
     end;
  end;
end;



procedure TForm1.Progress(currval, maxval: Int64);
const
  MAX_PROC : Byte = 100;
begin
if Assigned(Gauge1) then
  begin
    if (currval > 0) and (maxval > 0) then Gauge1.Progress := Round((currval/maxval)*MAX_PROC)
    else Gauge1.Progress := 0;
    Gauge1.MaxValue := MAX_PROC;
  end;
end;



procedure TForm1.AbortConnection;
begin
Display('# Abort connection...');
TcpClient1.Close;
FFileStream.Free; FFileStream := nil;
end;



function GetFileSize(FileName : string):int64;
var
  fs : TFileStream;
begin
  Result := 0;
  try
    if not FileExists(FileName) then Exit;
    fs := TFileStream.Create(FileName, fmOpenRead);
    Result := fs.Size;
  finally
    fs.Free; fs := nil;
  end;
end;


//for information
procedure TForm1.EditLineChange(Sender: TObject);
begin
LabelLength.Caption := IntToStr(Length(EditLine.text));
LabelBytes.Caption := IntToStr(Length(EditLine.text) + 2);
end;



procedure TForm1.Controls(Enabled: boolean);
begin
ButtonSendLine.Enabled := Enabled;
ButtonSendFile.Enabled := Enabled;
EditLine.Enabled := Enabled;
end;

end.
