unit UnitServer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ToolWin, ComCtrls, Gauges, Sockets, StrUtils, Math,
  XPMan, ImgList;

const
  CONST_BUFSIZE : Integer = 4096;
  CONST_DATATIMEOUT : integer = 1000;
  MARKER_LINE : string[4] = 'line';
  MARKER_FILE : string[4] = 'file';
  DestFolder : string[10] = 'downloads\';
  delim : string[1] = ':';


type
  TForm1 = class(TForm)
    Memo1: TMemo;
    Gauge1: TGauge;
    ToolBar1: TToolBar;
    lbl1: TLabel;
    EditPort: TEdit;
    ButtonStartStop: TToolButton;
    TcpServer1: TTcpServer;
    XPManifest1: TXPManifest;
    ImageList1: TImageList;
    procedure ButtonStartStopClick(Sender: TObject);
    procedure Display(const Msg : String);
    procedure TcpServer1Accept(Sender: TObject; ClientSocket: TCustomIpClient);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
    procedure Progress(currval, maxval: Int64);
    procedure AbortConnection(ClientSocket: TCustomIpClient);
  private
    FFileStream : TFileStream;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  LogCritSect  : TRTLCriticalSection;
  GLogList     : TStringList;

implementation

{$R *.dfm}

procedure TForm1.FormShow(Sender: TObject);
begin
//  ,      
if not DirectoryExists(DestFolder) then
   if not CreateDir(DestFolder) then
      begin
        Display('cant create dir: ' +QuotedStr(DestFolder));
        Application.Terminate;
      end;
end;



procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
if TcpServer1.Active then ButtonStartStop.Click;
end;



procedure TForm1.ButtonStartStopClick(Sender: TObject);
begin
try
  if EditPort.Enabled then TcpServer1.LocalPort := EditPort.Text;

  TcpServer1.Active := not TcpServer1.Active;

  if TcpServer1.Active then
    begin
      Display('Server started at port ' + QuotedStr(TcpServer1.LocalPort));
    end
  else
    begin
      Display('Server stoped');
      Progress(0, 0);
    end;

finally
  ButtonStartStop.Down := TcpServer1.Active;
  EditPort.Enabled := not TcpServer1.Active;
end;
end;



procedure TForm1.Display(const Msg : String);
begin
    //      
    //  BeginUpdate  EndUpdate
    Memo1.Lines.BeginUpdate;
    try
        Memo1.Lines.Add('['+TimeToStr(Time)+'] '+ Msg);
    finally
        //    
        Memo1.Perform(WM_VSCROLL, SB_BOTTOM, SB_THUMBTRACK);
        Memo1.Lines.EndUpdate;
    end;
end;



//    ,     .
//        

//          Accept  OnGetThread
//    ,        .
//   ,     1    
//      OnAccept,     

procedure TForm1.TcpServer1Accept(Sender: TObject; ClientSocket: TCustomIpClient);
var
  rcvdline, fName : string;
  fSize : Int64;
  buf : pointer;
  readCount, nRead: integer;
begin
Display('<Accept> from ' + QuotedStr(ClientSocket.RemoteHost + ':' + ClientSocket.RemotePort));

while True do
  begin
    //          
    rcvdline := ClientSocket.Receiveln;

    //   (   '' )
    if (rcvdline <> '') and ClientSocket.Connected then
      begin
        Display('|------------------------------');
        Display('| Accepted marker: ' + QuotedStr(rcvdline));

        {* * * * *}  //  
        if (rcvdline = MARKER_LINE) and ClientSocket.Connected then
          begin
            //  ,  
            rcvdline := ClientSocket.Receiveln;
            if Trim(rcvdline) = '' then
              begin
                 Display('| Dead line...');
                 Display('+------------------------------');
                 Exit;
              end
            else Display('| Received line: ' + QuotedStr(rcvdline));
            Display('| Received bytes: ' + IntToStr(ClientSocket.BytesReceived - length(MARKER_LINE) - 2));
            Display('| Received length: ' + IntToStr(length(rcvdline)));
            Display('+------------------------------');
          end //   

        {* * * * *}  // 
        else if (rcvdline = MARKER_FILE) and ClientSocket.Connected then
          begin

            //   3 : ,     ,
            //      3  'ClientSocket.Receiveln'
            // (1        2  )
            fName := ClientSocket.Receiveln;
            fSize := StrToInt64(ClientSocket.Receiveln);

            Display('| File name: ' + QuotedStr(fName));
            Display('| File size: ' + QuotedStr(IntToStr(fSize)));
            Display('| Receiving...');

            //     
            if FFileStream = nil then
               begin
                 try
                   FFileStream := TFileStream.Create(DestFolder + fName, fmCreate);
                   FFileStream.Position := 0;
                   //    0
                   Progress(0, 0);
                 except
                   on E: Exception do begin Display('#' + E.Message); Exit; end;
                 end;
               end;

             repeat
                //        
                // (   )
                if not ClientSocket.WaitForData(CONST_DATATIMEOUT) then
                  begin
                     AbortConnection(ClientSocket);
                     Exit;
                  end;

                //        ,
                //           

                // ,    4096,     
                //   500 ,     500 ,
                //  4096
                readCount := Min(fSize - FFileStream.Position, CONST_BUFSIZE);
                try
                  //     
                  GetMem(buf, readCount);
                  try
                    //     
                    //      readCount 
                    //     ...
                    nRead := ClientSocket.ReceiveBuf(buf^, readCount);
                  except
                    on E: Exception do begin Display('#' + E.Message); AbortConnection(ClientSocket); Exit; end;
                  end;

                  // ...    
                  if nRead > 0 then
                    begin
                       //     ,   (nRead),
                       //   readCount
                       FFileStream.WriteBuffer(buf^, nRead);
                       //  
                       Progress(FFileStream.Position, fSize);
                    end;
                finally
                  //  
                  FreeMem(buf, readCount);
                end;
             //    repeat -- until   ,   
             // FFileStream     fSize
             until FFileStream.Position = fSize;

             //   ...
             // (      repeat -- until  
             //       )
             if FFileStream.Position = fSize then
                 begin
                   FFileStream.Free; FFileStream := nil;
                   //     :
                   //    ,
                   //   -  ..
                   Display('| File ' + QuotedStr(fName) + ' received');
                   Display('+------------------------------');
                 end;
          end//  

       {* * * * *} //  
        else
          begin
            AbortConnection(ClientSocket);
          end;

      end
    //  rcvdline = ''  ,        
    else
      begin
        Display(QuotedStr(ClientSocket.RemoteHost + ':' + ClientSocket.RemotePort) + ' disconected.');
        break; //   'While True do'
      end;
  end;
end;



procedure TForm1.Progress(currval, maxval: Int64);
const
  MAX_PROC : Byte = 100;
begin
// ..         Gauge1.Progress (Integer),
//      
if Assigned(Gauge1) then
  begin
    if (currval > 0) and (maxval > 0) then Gauge1.Progress := Round((currval/maxval)*MAX_PROC)
    else Gauge1.Progress := 0;
    Gauge1.MaxValue := MAX_PROC;
  end;
end;


//       
procedure TForm1.AbortConnection(ClientSocket: TCustomIpClient);
begin
Display('# Abort connection...');
ClientSocket.Disconnect;
FFileStream.Free; FFileStream := nil;
end;

end.
